#!/usr/bin/perl
#
# Rubi's Ultimate Backup Implementation
#
# Version 2.0, (C) 3.1999 by Beat Rubischon <beat@rubis.ch>
#                  8.1999 by Beat Rubischon <beat.rubischon@trivadis.com>
#                  2.2000 by Beat Rubischon <beat@rubis.ch>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Main handler for backup

$SSH=$ARGV[0];
$CPIO=$ARGV[1];
$SRCCPIO=$ARGV[2];
$SRCHOST=$ARGV[3];
$SRCPART=$ARGV[4];
$BASEDST=$ARGV[5];
$DSTDIR=$ARGV[6];
$OLDDST=$ARGV[7];

$fccount=0; $fcbytes=0;
$flcount=0; $flbytes=0;
$scount=0;
$dcount=0;
$fcount=0;
$ccount=0;
$bcount=0;
$kcount=0;
$ucount=0;

$time=`date`;

print "Backup report\n";
print "----------------------------------------------------------------------------\n";
print "Backing up:             $SRCHOST:$SRCPART\n";
print "        to:             $DSTDIR\n";
print "last backup in:         ";
if ($OLDDST eq "") {
  print "no previus backup done\n";
} else {
  print "$OLDDST\n";
}
print "Start time:             $time";

if ($SRCHOST eq "localhost") {
  $walkcmd="/opt/rubi/lib/walk.pl ".$SRCPART."|";
  $cpiocmd="|".$CPIO." -p -u -m ".$DSTDIR;
} else {
  $walkcmd=$SSH." ".$SRCHOST." /opt/rubi/lib/walk.pl ".$SRCPART."|";
  $cpiocmd="|".$SSH." ".$SRCHOST." ".$SRCCPIO." -o -H newc | ".$CPIO." -i -u -m --no-absolute-filenames";
}

print "Open walk-channel:      $walkcmd\n";
print "Open cpio-channel:      $cpiocmd\n\n";

open(WALK, $walkcmd) || die "cant open";
open(CPIO, $cpiocmd) || die "cant open";

while(<WALK>) {
  chomp;
  ($type,$rdev,$rino,$rmode,$rnlink,$ruid,$rgid,$rrdev,$rsize,$ratime,$rmtime,$rctime,$rblksize,$rblocks,$file)=split(/,/);

  # regular file
  if ($type eq "10") {
    if ($OLDDST ne "") {
      ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks)=lstat($OLDDST.$file);
    }

    if ($OLDDST eq "" or $dev="" or $mtime ne $rmtime) {
      print CPIO "$file\n";
      $fccount++;
      $fcbytes=$fcbytes+$rsize;
    } else {
      link $OLDDST.$file,$DSTDIR.$file;
      $flcount++;
      $flbytes=$flbytes+$rsize;
    }
  }

  # symlink
  elsif($type eq "12") {
    print CPIO "$file\n";
    $scount++;
  }

  # directory
  elsif ($type eq "04") {
    @dirs=split(/\//,$file);
    $base=$DSTDIR;

    foreach(@dirs) {
      if ($_ ne "") {
        $base=$base."/".$_;
        if (stat($base) eq "") {
          mkdir($base, 0755);
        }
      }
    }

    chmod $rmode, $DSTDIR.$file;
    chown $ruid, $rgid, $DSTDIR.$file;
    $dcount++;  
  }

  # fifo
  elsif ($type eq "01") {
    print CPIO "$file\n";
    $fcount++;
  }
  
  # char special
  elsif ($type eq "02") {
    print CPIO "$file\n";
    $ccount++
  }
  
  # block special
  elsif ($type eq "06") {
    print CPIO "$file\n";
    $bcount++;
  }
  
  # socket
  elsif ($type eq "14") {
    print CPIO "$file\n";
    $kcount++;
  }
  
  # unknown
  else {
    print STDERR "$file has unknown type $type.\n";
    $ucount++;
  }
}

close(WALK);
close(CPIO);

$fcval="B";
if ($fcbytes >= 1024) {
  $fcbytes=$fcbytes / 1024;
  $fcval="K";
  if ($fcbytes >= 1024) {
    $fcbytes=$fcbytes / 1024;
    $fcval="M";
    if ($fcbytes >= 1024) {
      $fcbytes=$fcbytes / 1024;
      $fcval="G";
    }
  }
}

$flval="B";
if ($flbytes >= 1024) {
  $flbytes=$flbytes / 1024;
  $flval="K";
  if ($flbytes >= 1024) {
    $flbytes=$flbytes / 1024;
    $flval="M";
    if ($flbytes >= 1024) {
      $flbytes=$flbytes / 1024;
      $flval="G";
    }
  }
}

$time=`date`;
print "\nEnd time:               $time\n";
print "Type                    Count     Size\n";
print "----------------------------------------------------------------------------\n";
printf "%-22s %6i %6.1f %s\n", "files copied:", $fccount, $fcbytes, $fcval;
printf "%-22s %6i %6.1f %s\n", "files linked:", $flcount, $flbytes, $flval;
printf "%-22s %6i\n", "directories:", $dcount;
printf "%-22s %6i\n", "symlinks:", $scount;
printf "%-22s %6i\n", "char special:", $ccount;
printf "%-22s %6i\n", "block special:", $bcount;
printf "%-22s %6i\n", "fifos:", $fcount;
printf "%-22s %6i\n", "sockets:", $kcount;
printf "%-22s %6i\n", "unknown:", $ucount;
print "----------------------------------------------------------------------------\n";
print "\n";
